@@ -1,5 +1,6 @@
 `betadisper` <-
-    function(d, group, type = c("median","centroid"), bias.adjust=FALSE)
+    function(d, group, type = c("median","centroid"), bias.adjust=FALSE,
+             sqrt.dist = FALSE, add = FALSE)
 {
     ## inline function for double centring. We used .C("dblcen", ...,
     ## PACKAGE = "stats") which does not dublicate its argument, but
@@ -18,6 +19,18 @@
         spMedNeg <- ordimedian(vectors, group, choices = axes[!pos])
         cbind(spMedPos, spMedNeg)
     }
+    ## inline function for centroids
+    centroidFUN <- function(vec, group) {
+        cent <- apply(vec, 2,
+                      function(x, group) tapply(x, INDEX = group, FUN = mean),
+                      group = group)
+        if(!is.matrix(cent)) { ## if only 1 group, cent is vector
+            cent <- matrix(cent, nrow = 1,
+                           dimnames = list(as.character(levels(group)),
+                           paste0("Dim", seq_len(NCOL(vec)))))
+        }
+        cent
+    }
     ## inline function for distance computation
     Resids <- function(x, c) {
         if(is.matrix(c))
@@ -34,6 +47,22 @@
     ## Someone really tried to analyse correlation like object in range -1..+1
     if (any(d < -TOL, na.rm = TRUE))
         stop("dissimilarities 'd' must be non-negative")
+    ## adjust to avoid negative eigenvalues (if they disturb you)
+    if (sqrt.dist)
+        d <- sqrt(d)
+    if (is.logical(add) && isTRUE(add))
+        add <- "lingoes"
+    if (is.character(add)) {
+        add <- match.arg(add, c("lingoes", "cailliez"))
+        if (add == "lingoes") {
+            ac <- addLingoes(as.matrix(d))
+            d <- sqrt(d^2 + 2 * ac)
+        }
+        else if (add == "cailliez") {
+            ac <- addCailliez(as.matrix(d))
+            d <- d + ac
+        }
+    }
     if(missing(type))
         type <- "median"
     type <- match.arg(type)
@@ -56,7 +85,7 @@
         n <- n - sum(gr.na)
         ## update labels
         labs <- labs[!gr.na]
-        warning("Missing observations due to 'group' removed.")
+        warning("missing observations due to 'group' removed")
     }
     ## remove NA's in d
     if(any(x.na <- apply(x, 1, function(x) any(is.na(x))))) {
@@ -66,10 +95,9 @@
         n <- n - sum(x.na)
         ## update labels
         labs <- labs[!x.na]
-        warning("Missing observations due to 'd' removed.")
+        warning("missing observations due to 'd' removed")
     }
     x <- x + t(x)
-    storage.mode(x) <- "double"
     x <- dblcen(x)
     e <- eigen(-x/2, symmetric = TRUE)
     vectors <- e$vectors
@@ -84,7 +112,7 @@
     ## group centroids in PCoA space
     centroids <-
         switch(type,
-               centroid = apply(vectors, 2, function(x) tapply(x, group, mean)),
+               centroid = centroidFUN(vectors, group),
                median = spatialMed(vectors, group, pos)
                )
     ## for each of the groups, calculate distance to centroid for
