@@ -20,7 +20,7 @@
                  "co"="(a*c+a*b+2*b*c)/(2*(a+b)*(a+c))",
                  "cc"="(b+c)/(a+b+c)", "g"="(b+c)/(a+b+c)",
                  "-3"="pmin(b,c)/(a+b+c)", "l"="(b+c)/2",
-                 "19"="2*(b*c+1)/((a+b+c)^2-(a+b+c))",
+                 "19"="2*(b*c+1)/(a+b+c)/(a+b+c-1)",
                  "hk"="(b+c)/(2*a+b+c)", "rlb"="a/(a+c)",
                  "sim"="pmin(b,c)/(pmin(b,c)+a)",
                  "gl"="2*abs(b-c)/(2*a+b+c)",
