@@ -38,13 +38,12 @@
     inertia <- attr(X, "method")
     if (is.null(inertia))
         inertia <- "unknown"
-    inertia <- paste(toupper(substr(inertia, 1, 1)), substr(inertia, 
-                                                            2, 256), sep = "")
+    inertia <- paste(toupper(substr(inertia, 1, 1)),
+                     substring(inertia,  2), sep = "")
     inertia <- paste(inertia, "distance")
     if (!sqrt.dist)
         inertia <- paste("squared", inertia)
-    if (add) 
-        inertia <- paste(inertia, "(euclidified)")
+    ## postpone info on euclidification till we have done so
 
     ## evaluate formula: ordiParseFormula will return dissimilarities
     ## as a symmetric square matrix (except that some rows may be
@@ -76,17 +75,13 @@
         adjust <- sqrt(k)
     }
     nm <- attr(X, "Labels")    
-    ## cmdscale is only used if 'add = TRUE': it cannot properly
-    ## handle negative eigenvalues and therefore we normally use
-    ## wcmdscale. If we have 'add = TRUE' there will be no negative
-    ## eigenvalues and this is not a problem.
-    if (add) {
-        X <- cmdscale(X, k = k, eig = TRUE, add = add)
-        ## All eigenvalues *should* be positive, but see that they are
-        X$eig <- X$eig[X$eig > 0]
-    }
-    else
-        X <- wcmdscale(X, eig = TRUE)
+    ## wcmdscale, optionally with additive adjustment
+    X <- wcmdscale(X, x.ret = TRUE, add = add)
+    ## this may have been euclidified: update inertia
+    if (!is.na(X$ac) && X$ac > sqrt(.Machine$double.eps))
+        inertia <- paste(paste0(toupper(substring(X$add, 1, 1)),
+                                substring(X$add, 2)),
+                         "adjusted", inertia)
     if (is.null(rownames(X$points))) 
         rownames(X$points) <- nm
     X$points <- adjust * X$points
@@ -98,6 +93,39 @@
             X$negaxes <- X$negaxes/sqrt(k)
     }
     sol <- rda.default(X$points, d$Y, d$Z, ...)
+    ## Get components of inertia with negative eigenvalues following
+    ## McArdle & Anderson (2001), section "Theory". G is their
+    ## double-centred Gower matrix, but instead of hat matrix, we use
+    ## use QR decomposition to get the components of inertia.
+    hasNegEig <- any(X$eig < 0)
+    G <- -X$x/2
+    if (adjust == 1)
+        G <- G/k
+    if (hasNegEig)
+        sol$real.tot.chi <- sol$tot.chi
+    sol$tot.chi <- sum(diag(G))
+    if (!is.null(sol$pCCA)) {
+        sol$pCCA$G <- G
+        if (hasNegEig) {
+            sol$pCCA$real.tot.chi <- sol$pCCA$tot.chi
+            sol$pCCA$tot.chi <- sum(diag(qr.fitted(sol$pCCA$QR, G)))
+        }
+        G <- qr.resid(sol$pCCA$QR, t(qr.resid(sol$pCCA$QR, G)))
+    }
+    if (!is.null(sol$CCA) && sol$CCA$rank > 0) {
+        sol$CCA$G <- G
+        if (hasNegEig) {
+            sol$CCA$real.tot.chi <- sol$CCA$tot.chi
+            sol$CCA$tot.chi <- sum(diag(qr.fitted(sol$CCA$QR, G)))
+        }
+    }
+    if (hasNegEig) {
+        sol$CA$real.tot.chi <- sol$CA$tot.chi
+        if (!is.null(sol$CA) && !is.null(sol$CCA$QR))
+            sol$CA$tot.chi <- sum(diag(qr.resid(sol$CCA$QR, G)))
+        else
+            sol$CA$tot.chi <- sum(diag(G))
+    }
     if (!is.null(sol$CCA) && sol$CCA$rank > 0) {
         colnames(sol$CCA$u) <- colnames(sol$CCA$biplot) <- names(sol$CCA$eig) <-
             colnames(sol$CCA$wa) <- colnames(sol$CCA$v) <-
@@ -112,7 +140,6 @@
     if (any(X$eig < 0)) {
         negax <- X$eig[X$eig < 0]
         sol$CA$imaginary.chi <- sum(negax)
-        sol$tot.chi <- sol$tot.chi + sol$CA$imaginary.chi
         sol$CA$imaginary.rank <- length(negax)
         sol$CA$imaginary.u.eig <- X$negaxes
     }
@@ -161,8 +188,11 @@
     sol$call$formula <- formula(d$terms, width.cutoff = 500)
     sol$call$formula[[2]] <- formula[[2]]
     sol$method <- "capscale"
-    if (add)
+    sol$sqrt.dist <- sqrt.dist
+    if (!is.na(X$ac) && X$ac > 0) {
         sol$ac <- X$ac
+        sol$add <- X$add
+    }
     sol$adjust <- adjust
     sol$inertia <- inertia
     if (metaMDSdist)
