@@ -1,6 +1,6 @@
 `scores.cca` <-
-    function (x, choices = c(1, 2), display = c("sp", "wa", "cn"), 
-              scaling = 2, ...) 
+    function (x, choices = c(1, 2), display = c("sp", "wa", "cn"),
+              scaling = "species", hill = FALSE, ...)
 {
     if(inherits(x, "pcaiv")) {
         warning("looks like ade4::cca object: you better use ade4 functions")
@@ -10,10 +10,10 @@
     ## "exclude"
     if (!is.null(x$na.action) && inherits(x$na.action, "exclude"))
         x <- ordiNApredict(x$na.action, x)
-    tabula <- c("species", "sites", "constraints", "biplot", 
+    tabula <- c("species", "sites", "constraints", "biplot",
                 "centroids")
     names(tabula) <- c("sp", "wa", "lc", "bp", "cn")
-    if (is.null(x$CCA)) 
+    if (is.null(x$CCA))
         tabula <- tabula[1:2]
     display <- match.arg(display, c("sites", "species", "wa",
                                     "lc", "bp", "cn"),
@@ -26,6 +26,8 @@
     slam <- sqrt(c(x$CCA$eig, x$CA$eig)[choices])
     rnk <- x$CCA$rank
     sol <- list()
+    ## process scaling; numeric scaling will just be returned as is
+    scaling <- scalingType(scaling = scaling, hill = hill)
     if ("species" %in% take) {
         v <- cbind(x$CCA$v, x$CA$v)[, choices, drop = FALSE]
         if (scaling) {
@@ -64,18 +66,18 @@
     }
     if ("biplot" %in% take && !is.null(x$CCA$biplot)) {
         b <- matrix(0, nrow(x$CCA$biplot), length(choices))
-        b[, choices <= rnk] <- x$CCA$biplot[, choices[choices <= 
+        b[, choices <= rnk] <- x$CCA$biplot[, choices[choices <=
             rnk]]
         colnames(b) <- c(colnames(x$CCA$u), colnames(x$CA$u))[choices]
         rownames(b) <- rownames(x$CCA$biplot)
         sol$biplot <- b
     }
     if ("centroids" %in% take) {
-        if (is.null(x$CCA$centroids)) 
+        if (is.null(x$CCA$centroids))
             sol$centroids <- NA
         else {
             cn <- matrix(0, nrow(x$CCA$centroids), length(choices))
-            cn[, choices <= rnk] <- x$CCA$centroids[, choices[choices <= 
+            cn[, choices <= rnk] <- x$CCA$centroids[, choices[choices <=
                  rnk]]
             colnames(cn) <- c(colnames(x$CCA$u), colnames(x$CA$u))[choices]
             rownames(cn) <- rownames(x$CCA$centroids)
@@ -93,14 +95,14 @@
     ## Take care that scores have names
     if (length(sol)) {
         for (i in seq_along(sol)) {
-            if (is.matrix(sol[[i]])) 
+            if (is.matrix(sol[[i]]))
                 rownames(sol[[i]]) <-
-                    rownames(sol[[i]], do.NULL = FALSE, 
+                    rownames(sol[[i]], do.NULL = FALSE,
                              prefix = substr(names(sol)[i], 1, 3))
         }
     }
     ## Only one type of scores: return a matrix instead of a list
-    if (length(sol) == 1) 
+    if (length(sol) == 1)
         sol <- sol[[1]]
     sol
 }
